(ns printer
  (:require [cljs.nodejs :as node]))

"This printer requires in the data map:
:content (can be a single item or a list)
    -> if single item, then add {:print-type :get} to the data
    -> if list, then add {:print-type :list} to the data
:outcome -> nice message that will be shown on top of the table ('User Updated!')
:print-type [:get :list]
    -> :get will print key value attribute table
    -> :list will dynamically create columns per attribute. Require extra tags:
        -> :table-head [\"id\" \"name\" \"my-field\"]
        -> :columns-width [5 10 20]"

(def Table (node/require "cli-table"))

(defn print-outcome [body]
  (js/console.log (str "\n" (:outcome body)))
  body)

(defn fill [table data]
  (doall (map #(.push table (clj->js %)) data)))

(defmulti build-table (fn [data] (:print-type data)))
(defmethod build-table :get [data]
  (let [table (Table. (clj->js {:head ["Attribute" "Value"]
                                :colWidths [20, 60]}))
        table (doto table (fill (:content data)))]
    (assoc data :table table)))

(defmethod build-table :list [data]
  (let [table (Table. (clj->js {:head (:table-head data)
                                :colWidths (:columns-width data)}))
        table (doto table (fill (:content data)))]
    (assoc data :table table)))

(defn print-table [data]
  (-> (:table data) .toString js/console.log))

(defn replace-nil-to-blank-in-map [data]
  (into {} (map #(if (second %) % {(first %) ""}) data)))

(defn replace-nil-to-blank-in-seq [data]
  (map #(if (nil? %) "" %) data))

(defmulti remove-nils (fn [data] (:print-type data)))
(defmethod remove-nils :list [data]
  (let [content (:content data)
        content (map replace-nil-to-blank-in-seq content)]
    (assoc data :content content)))

(defmethod remove-nils :get [data]
  (assoc data :content (replace-nil-to-blank-in-map (:content data))))

(defn print-data [data]
  (-> data
      remove-nils
      build-table
      print-outcome
      print-table))
